#! /usr/bin/env python
# Semplice script per modificare su LDAP i dati di
# scadenza delle password
#
# Copyright (C) 2005 Simone Piccardi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import ldap
from os import *
import re
import getopt, sys


# Definitios
attributes = { '-m': 'shadowMin',
               '-M': 'shadowMax',
               '-d': 'shadowLastChange',
               '-E': 'shadowExpire',
               '-I': 'shadowInactive',
               '-W': 'shadowWarning' }
listvalues = False

# Command line argument processing
try:
    opts, args = getopt.getopt(sys.argv[1:], 'm:M:d:E:I:W:l') 
    #print opts, args
    modif = []
    for opt, val in opts:
        if opt != '-l':
            #print opt, attributes[opt]
            modif.append( (ldap.MOD_REPLACE, attributes[opt], val) )
        else:
            listvalues = True

    #print modif

except getopt.GetoptError:
    print "Usage: chage.py [-l] [-m min_days] [-M max_days] [-W warn]"
    print "     [-I inactive] [-E expire] [-d last_day] user"
    sys.exit(2)
        
# LDAP connection
try: 
    l = ldap.open("127.0.0.1")
#    l=ldap.initialize("ldaps://ldap.truelite.it")
    l.protocol_version = ldap.VERSION3
    username = "cn=admin,dc=domain,dc=local"
    password = "<PASSWORD>"
    l.simple_bind(username, password)
except ldap.LDAPError, e:
    print "Connection error", e

# LDAP query
baseDN = "dc=domain,dc=local"
searchScope = ldap.SCOPE_SUBTREE
retrieveAttributes = attributes.values()

# loop over the users in args
for user in args:
    searchFilter = "uid="+user
    try:
        ldap_result_id = l.search(baseDN, searchScope, searchFilter,
                                  retrieveAttributes)
        result_set = []
        count = 0
        while 1:
            result_type, result_data = l.result(ldap_result_id, 0)
            if (result_data == []):
                break
            else:
                if result_type == ldap.RES_SEARCH_ENTRY:
                    result_set.append(result_data)

    except ldap.LDAPError, e:
        print "Search error", e
    
    if len(result_set) == 0:
        print "User", user, "unknown"
        continue


    if listvalues:
        for i in range(len(result_set)):
            for voce in result_set[i]:
                print user, voce[0]
                print "\t ->", voce[1]

    if modif:
        for i in range(len(result_set)):
            for voce in result_set[i]:
                dn = voce[0]
                print user,',', dn
                print "\t ->", modif
                l.modify_s(dn, modif)


